@@ -1,7 +1,5 @@
-from tileserver.leaflet import (  # noqa
-    get_leaflet_tile_layer,
-)
-from tileserver.report import Report  # noqa
-from tileserver.server import TileServer  # noqa
-from tileserver.utilities import get_cache_dir  # noqa
 from tileserver._version import __version__  # noqa
+from tileserver.leaflet import get_leaflet_tile_layer  # noqa
+from tileserver.report import Report  # noqa
+from tileserver.server import TileClient  # noqa
+from tileserver.utilities import get_cache_dir  # noqa
