@@ -1,28 +1,32 @@
 import logging
 import pathlib
-import requests
 import threading
+from typing import Union
+
+import requests
 from werkzeug.serving import make_server
 
-from tileserver.utilities import save_file_from_request
-from tileserver.application.paths import inject_path, pop_path
+from tileserver.utilities import add_query_parameters, save_file_from_request
+
+_LIVE_SERVERS = {}
 
 
-def run_app(path: pathlib.Path, port: int = 0, debug: bool = False):
-    from tileserver.application import app
-
-    path = pathlib.Path(path).expanduser()
-    inject_path("default", path)
-    app.config["DEBUG"] = debug
-    return app.run(host="localhost", port=port)
+logger = logging.getLogger(__name__)
 
 
-class TileServerThred(threading.Thread):
+class TileServerThread(threading.Thread):
     """This is for internal use only."""
 
-    def __init__(self, path: pathlib.Path, port: int = 0, debug: bool = False):
+    class ServerDownError(Exception):
+        """Raised when a TileServerThread is down."""
+
+        pass
+
+    def __init__(self, port: int = 0, debug: bool = False, start: bool = True):
+        if not isinstance(port, int):
+            raise ValueError(f"Port must be an int, not {type(port)}")
+
         threading.Thread.__init__(self)
-        path = pathlib.Path(path).expanduser()
 
         from tileserver.application import app
 
@@ -32,28 +36,67 @@ class TileServerThred(threading.Thread):
             logging.getLogger("large_image").setLevel(logging.ERROR)
         else:
             app.config["DEBUG"] = True
+            logging.getLogger("werkzeug").setLevel(logging.DEBUG)
+            logging.getLogger("gdal").setLevel(logging.DEBUG)
+            logging.getLogger("large_image").setLevel(logging.DEBUG)
 
         self.daemon = True  # CRITICAL for safe exit
         self.srv = make_server("localhost", port, app)
         self.ctx = app.app_context()
         self.ctx.push()
-        self.path = path
+        if start:
+            self.start()
 
     def run(self):
-        # This is absolutely critical this happens here
-        inject_path(self.ident, self.path)
         self.srv.serve_forever()
 
     def shutdown(self):
         if self.is_alive():
             self.srv.shutdown()
-            pop_path(self.ident)
 
     def __del__(self):
         self.shutdown()
 
+    @property
+    def port(self):
+        return self.srv.port
 
-class TileServer:
+    @property
+    def host(self):
+        return self.srv.host
+
+
+def is_server_live(key: Union[int, str]):
+    return key in _LIVE_SERVERS and _LIVE_SERVERS[key].is_alive()
+
+
+def launch_server(port: Union[int, str] = "default", debug: bool = False):
+    if is_server_live(port):
+        return port
+    if port == "default":
+        server = TileServerThread(0, debug)
+    else:
+        server = TileServerThread(port, debug)
+        if port == 0:
+            # Get reallocated port
+            port = server.port
+    _LIVE_SERVERS[port] = server
+    return port
+
+
+def shutdown_server(key: int, force: bool = False):
+    if not force and key == "default":
+        # We do not shut down the default server
+        return
+    try:
+        server = _LIVE_SERVERS.pop(key)
+        server.shutdown()
+        del server
+    except KeyError:
+        logger.error(f"Server for key ({key}) not found.")
+
+
+class TileClient:
     """Serve tiles from a local raster file in a background thread.
 
     Parameters
@@ -68,41 +111,58 @@ class TileServer:
 
     """
 
-    def __init__(self, path: pathlib.Path, port: int = 0, debug: bool = False):
-        self._server = TileServerThred(path, port, debug)
-        self._server.start()  # run app threaded
-        # Be careful if assigning any other attributes to avoid cycilical refs
+    def __init__(
+        self,
+        filename: pathlib.Path,
+        port: Union[int, str] = "default",
+        debug: bool = False,
+    ):
+        path = pathlib.Path(filename).expanduser().absolute()
+        if not path.exists():
+            raise OSError(f"Source file path does not exist: {path}")
+        self._filename = path
+        self._key = launch_server(port, debug)
+        # Store actual port just in case
+        self._port = _LIVE_SERVERS[self._key].srv.port
 
     def __del__(self):
         self.shutdown()
 
     @property
-    def server(self):
-        return self._server
+    def filename(self):
+        return self._filename
 
     @property
-    def path(self):
-        return self.server.path
+    def server(self):
+        try:
+            return _LIVE_SERVERS[self._key]
+        except KeyError:
+            raise TileServerThread.ServerDownError(
+                "Tile server for this source has been shutdown."
+            )
 
     @property
     def port(self):
-        return self.server.srv.port
+        return self.server.port
 
     @property
     def base_url(self):
-        return f"http://{self.server.srv.host}:{self.port}"
+        return f"http://{self.server.host}:{self.port}"
 
-    def shutdown(self):
-        self.server.shutdown()
+    def shutdown(self, force: bool = False):
+        shutdown_server(self._key, force=force)
+
+    def _produce_url(self, base: str):
+        return add_query_parameters(base, {"filename": self._filename})
 
     def create_url(self, path: str):
-        return f"{self.base_url}/{path.lstrip('/')}"
+        return self._produce_url(f"{self.base_url}/{path.lstrip('/')}")
 
     def get_tile_url(self, projection: str = "EPSG:3857"):
-        url = self.create_url(f"tiles/{{z}}/{{x}}/{{y}}.png?")
-        if projection:
-            url += f"projection={projection}"
-        return url
+        url = add_query_parameters(
+            self.create_url("__tileserver_path__"), {"projection": projection}
+        )
+        return url.replace("__tileserver_path__", "tiles/{z}/{x}/{y}.png")
 
     def extract_roi(
         self,
